(*
 * Copyright 2009-2010, University of Copenhagen
 * Nicolas Palix
 * This file is part of Herodotos.
 *
 * Herodotos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, according to version 2 of the License.
 *
 * Herodotos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Herodotos.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The authors reserve the right to distribute this or future versions of
 * Herodotos under other licenses.
 *)


open Lexing

exception BadFormat
exception Misconfiguration of string

let parse_csv file =
  let in_ch = open_in file in
  let lexbuf = Lexing.from_channel in_ch  in
  try
    (* ignore(Parsing.set_trace true); *)
    Misc.init file lexbuf;
    let ast = Exist_parser.entries Exist_lexer.token lexbuf in
      close_in in_ch;
      ast
  with
      (Exist_lexer.Lexical msg) ->
	let pos = lexbuf.lex_curr_p in
	  Misc.print_error
	    { Ast.file  = file;
	      Ast.line  = pos.pos_lnum + 1;
	      Ast.colfr = pos.pos_cnum - pos.pos_bol;
	      Ast.colto = (Lexing.lexeme_end lexbuf) - pos.pos_bol + 1}
	    ("CSV Lexer Error: " ^ msg);
	  raise BadFormat
    | Exist_parser.Error ->
	let pos = lexbuf.lex_curr_p in
	  Misc.print_error
	    { Ast.file  = file;
	      Ast.line  = pos.pos_lnum + 1;
	      Ast.colfr = pos.pos_cnum - pos.pos_bol;
	      Ast.colto = (Lexing.lexeme_end lexbuf) - pos.pos_bol + 1}
	    ("CSV Parser Error: unexpected token '" ^ (Lexing.lexeme lexbuf) ^"'");
	  raise BadFormat


let int_of_state s =
  match s with
    Ast_exist.Empty  -> raise BadFormat
  | Ast_exist.True   -> raise BadFormat
  | Ast_exist.False  -> raise BadFormat
  | Ast_exist.Size i -> i

