(*
 * Copyright 2009-2010, University of Copenhagen
 * Nicolas Palix
 * This file is part of Herodotos.
 *
 * Herodotos is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, according to version 2 of the License.
 *
 * Herodotos is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Herodotos.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The authors reserve the right to distribute this or future versions of
 * Herodotos under other licenses.
 *)


type version = string * Unix.tm * int

type correl = Strict | Nocorrel | Default
type format = CSV | Org

type expression =
    Pattern of string
  | Project of string
  | Cst     of float
  | Plus  of expression * expression
  | Minus of expression * expression
  | Mul   of expression * expression
  | Div   of expression * expression

type attr =
    Author of bool
  | CleanColor of float * float * float
  | Color of float * float * float
  | Correl of string
  | Data of expression
  | SQLData of string
  | DftPattern of string
  | DftProject of string
  | Dir of string
  | Factor of float
  | File of string option
  | Filename of bool
  | Flags of string
  | Footer of string
  | Format of string
  | Info of bool
  | Legend of string
  | LineType of string
  | MarkSize of float
  | MarkType of string
  | NotExistColor of float * float * float
  | PatternColor of float * float * float
  | Prune of bool
  | Ratio of bool
  | SCM of string
  | Size of float * float
  | SubDir of string
  | VMin of string
  | Version of int * version list
  | XAxis of string
  | XLabel of string
  | YLabel of string
  | XLegend of string
  | XMin of float
  | YMin of float
  | YMax of float
  | YAxis of string
  | YLegend of string
  | YLegendFactor of string

type curve = string option * string option * attr list * Ast.pos

type group =
    GrpCurve of string * curve list
  | GrpPatt of string * Ast.pos

type subgraph =
    Curves of curve list
  | Groups of group list

type gph = attr list * subgraph
